import aiohttp
from .exceptions import EOLTrackerAPIError
from typing import List, Dict

class EOLClient:
    def __init__(self, session):
        self._session = session

    async def fetch_release_data(self, uri: str) -> dict:
        try:
            async with self._session.get(uri) as release_resp:
                if release_resp.status != 200:
                    raise EOLTrackerAPIError(
                        f"Failed to fetch release data from {uri}",
                        status_code=release_resp.status,
                        payload={"uri": uri}
                    )
                return (await release_resp.json()).get("result", {})
        except aiohttp.ClientError as e:
            raise EOLTrackerAPIError(
                f"HTTP error while fetching release data: {e}",
                payload={"uri": uri}
            )

    async def fetch_product_data(self, uri: str) -> dict:
        base_uri = "/".join(uri.strip("/").split("/")[:-2])
        try:
            async with self._session.get(base_uri) as product_resp:
                if product_resp.status != 200:
                    raise EOLTrackerAPIError(
                        f"Failed to fetch product data from {base_uri}",
                        status_code=product_resp.status,
                        payload={"uri": base_uri}
                    )
                return (await product_resp.json()).get("result", {})
        except aiohttp.ClientError as e:
            raise EOLTrackerAPIError(
                f"HTTP error while fetching product data: {e}",
                payload={"uri": base_uri}
            )

    async def fetch_all(self, uri: str) -> dict:
        release_data = await self.fetch_release_data(uri)
        product_data = await self.fetch_product_data(uri)
        return {
            "release": release_data,
            "product": product_data
        }

    async def fetch_all_products(self) -> List[dict]:
        uri = "https://endoflife.date/api/v1/products"
        async with self._session.get(uri) as resp:
            if resp.status != 200:
                raise EOLTrackerAPIError(
                    "Failed to fetch products",
                    status_code=resp.status,
                    payload={"uri": uri}
                )

            return (await resp.json()).get("result", [])

    async def fetch_product_versions(self, product_name: str) -> Dict[str, str]:
        uri = f"https://endoflife.date/api/v1/products/{product_name}"
        async with self._session.get(uri) as resp:
            if resp.status != 200:
                raise EOLTrackerAPIError(
                    f"Failed to fetch product versions for {product_name}",
                    status_code=resp.status,
                    payload={"uri": uri}
                )
            data = await resp.json()
            releases = data.get("result", {}).get("releases", [])
            return {
                release.get("label"): release.get("name")
                for release in releases if release.get("label") and release.get("name")
            }

    async def close(self):
        await self._session.close()


import pytest
from aioresponses import aioresponses
from aiohttp import ClientSession
from eoltracker.api import EOLClient
from eoltracker.exceptions import EOLTrackerAPIError

RELEASE_URL = "https://endoflife.date/api/v1/products/iPad/releases/11"
PRODUCT_URL = "https://endoflife.date/api/v1/products/iPad"

release_payload = {
    "result": {
        "label": "iPadOS 11",
        "isLts": False,
        "releaseDate": "2017-09-19"
    }
}

product_payload = {
    "result": {
        "label": "iPad",
        "links": {
            "html": "https://endoflife.date/ipad",
            "icon": "https://example.com/icon.png"
        }
    }
}

@pytest.mark.asyncio
async def test_fetch_release_data_success():
    with aioresponses() as m:
        m.get(RELEASE_URL, payload=release_payload)
        async with ClientSession() as session:
            client = EOLClient(session)
            data = await client.fetch_release_data(RELEASE_URL)
            assert data["label"] == "iPadOS 11"

@pytest.mark.asyncio
async def test_fetch_product_data_success():
    with aioresponses() as m:
        m.get(PRODUCT_URL, payload=product_payload)
        async with ClientSession() as session:
            client = EOLClient(session)
            data = await client.fetch_product_data(RELEASE_URL)
            assert data["label"] == "iPad"

@pytest.mark.asyncio
async def test_fetch_all_success():
    with aioresponses() as m:
        m.get(RELEASE_URL, payload=release_payload)
        m.get(PRODUCT_URL, payload=product_payload)
        async with ClientSession() as session:
            client = EOLClient(session)
            data = await client.fetch_all(RELEASE_URL)
            assert "release" in data and "product" in data

@pytest.mark.asyncio
async def test_fetch_release_data_error():
    with aioresponses() as m:
        m.get(RELEASE_URL, status=404)
        async with ClientSession() as session:
            client = EOLClient(session)
            with pytest.raises(EOLTrackerAPIError):
                await client.fetch_release_data(RELEASE_URL)

@pytest.mark.asyncio
async def test_fetch_product_data_error():
    with aioresponses() as m:
        m.get(PRODUCT_URL, status=500)
        async with ClientSession() as session:
            client = EOLClient(session)
            with pytest.raises(EOLTrackerAPIError):
                await client.fetch_product_data(RELEASE_URL)



# exception for EOL Tracker related error
class EOLTrackerAPIError(Exception):
    def __init__(self, message, status_code=None, payload=None):
        super().__init__(message)
        self.message = message
        self.status_code = status_code
        self.payload = payload

    def __str__(self):
        base = f"EOLTrackerAPIError: {self.message}"
        if self.status_code:
            base += f" (status code: {self.status_code})"
        if self.payload:
            base += f" | Payload: {self.payload}"
        return base